@@ -1,4 +1,76 @@
 
+def cgss_sequence(ts, n = None, tau = None, B = 10, 
+               embedding_method = 'standard', binning_method = 'equal_frequency'):
+    """This function creates a coarse grained state space (cgss) network represented as an adjacency matrix A using a 1-D time series
+    
+    Args:
+        ts (1-D array): 1-D time series signal
+    
+    Other Parameters:
+        n (Optional[int]): embedding dimension for state space reconstruction. Default is uses false nearest negihbor algorithm from parameter_selection module.
+        tau (Optional[int]): embedding delay from state space reconstruction. Default uses mutual information algorithm from parameter_selection module.
+        B (Optional[int]): number of bins per dimension for graph formation. Default is 10.
+        embedding_method (Optional[string]): embedding method as "standard" state space reconstruction or embedding of differences as "difference".
+        binning_method (Optional[string]): "equal_frequency" or "equal_size" binning options for state space.
+        
+    Returns:
+        [2-D square array]: A (2-D weighted and directed square adjacency matrix)
+    """
+    #import sub modules
+    import os
+    import sys
+    sys.path.insert(0, os.path.join(os.path.dirname(__file__),'..','..'))
+    sys.path.insert(0, os.path.join(os.path.dirname(__file__),'..'))
+    from teaspoon.SP import tsa_tools
+    import numpy as np
+    
+    def equalObs(x, B): #define function to calculate equal-frequency bins based on interpolation
+        return np.interp(np.linspace(0, len(x), B), np.arange(len(x)), np.sort(x))
+    
+    #----------------Get embedding parameters if not defined----------------
+    if tau == None:
+        from parameter_selection import MI_delay
+        tau = MI_delay.MI_for_delay(ts, method = 'basic', h_method = 'sturge', k = 2, ranking = True)
+    if n == None:
+        from parameter_selection import FNN_n
+        perc_FNN, n = FNN_n.FNN_n(ts, tau)
+    
+    #get state space reconstruction from signal (SSR)
+    SSR = tsa_tools.takens(ts, n, tau) 
+    
+    
+    #----------------Define how to use the embedding----------------
+    if embedding_method == 'difference': #uses standard state space reconstruction
+        delta = np.diff(SSR, axis = 1) #get differences in SSR vectors along coordinate axis
+        delta = delta.T #transpose delta array to put in columns
+        embedding = delta
+        N = B**n
+        basis = B**(np.arange(n-1)) #basis for assigning symbolic value
+    if embedding_method == 'standard': #uses differences along axis of state space reconstruction
+        embedding  = np.array(SSR).T
+        N = B**(n-1)
+        basis = B**(np.arange(n)) #basis for assigning symbolic value
+        
+        
+    #----------------Define how to partition the embedding----------------
+    if binning_method == 'equal_frequency':
+        #define bins with equal-frequency or probability (approximately) 
+        B_array = equalObs(embedding.flatten(), B+1)
+        B_array[-1] = B_array[-1]*(1 + 10**-10)
+    if binning_method == 'equal_size': #define bins based on equal spacing
+        B_array = np.linspace(np.amin(embedding), np.amax(embedding)*(1+10**-10), B+1) 
+        
+        
+    #----------------digitize the embedding to a sequence----------------
+    digitized_embedding = [] #prime the digitized version of deltas
+    for e_i in embedding: #nloop through n-1 delta positions
+        digitzed_vector = np.digitize(e_i, bins = B_array) # digitalize column delta_i
+        digitized_embedding.append(digitzed_vector) #append to digitalized deltas data structure
+    digitized_embedding = np.array(digitized_embedding).T - 1 #digitalize and stacked delta vectors
+    symbol_seq = np.sum(np.array(basis)*digitized_embedding, axis = 1) # symbolic sequence from basis and D
+    
+    
+    return symbol_seq
 
 # In[ ]:
     
